#!/usr/bin/env node
const path = require('path')
const { spawn, execSync, execFileSync, exec } = require('child_process')
const merge = require('merge')
const fs = require('fs-extra')
const AlipayFlow = require('./alipay-flow')
const WeappFlow = require('./weapp-flow')
const sendDing = require('./send-ding')

const envs = ['dev', 'test', 'mo', 'prod']

const {argv} = require('yargs')
  .option('config', {
    default: './deploy-config.js',
    describe: '配置文件路径'
  })

const log = console.log

//拉取分支
function fetchBranch(branch) {
  log('开始拉取分支')
  let _localBranch = branch
  if (branch.length > 15) {
    _localBranch = branch.slice(15)
  }
  return new Promise((resolve, reject) => {
    exec(`git pull && git checkout ${_localBranch}`, function (err) { 
      if (err == null) {
        log(`拉取分支成功 --- ${branch}`)
        resolve(true)
      } else {
        reject(err)
      }
    })      
  })
}

//切换环境
function fetchEnv(env) {
  log(`开始切换环境`)
  return new Promise((resolve, reject) => {
    exec(`yarn env ${env}`, function(err, stdout, stderr) {
      if (err == null) {
        log(`切换环境成功 --- ${env}`)
        resolve(true)
      } else {
        log(err)
        reject(err)
      }
    })
  })
}

function pullBranch() {
  return new Promise(async (resolve, reject) => {
    let buildOptioons = process.argv.splice(2)
    if (buildOptioons.length < 2) {
      log('请输入完整的分支、环境')
      resolve(false)
      return
    }
    let branch = buildOptioons[0]
    let env = buildOptioons[1]
    if (!envs.includes(env)) {
      log('请输入正确的环境')
      resolve(false)
      return
    }
    let isSucc = await fetchBranch(branch)
  
    if (!isSucc) {
      log('切换分支失败')
      resolve(false)
      return
    }
  
    let envSu = await fetchEnv(env)
  
    if (!envSu) {
      log('切换环境失败')
      resolve(false)
      return
    } else {
      resolve(true)
    }

  })
}



function getAbsPath(p) {
  if (path.isAbsolute(p)) return p
  return path.join(process.cwd(), p)
}




// async function sendDingMsg(alipayQRImgUrl, weappQRImgUrl) {
//   // if (!alipayQRImgUrl && !weappQRImgUrl) {
//   //   log('缺少二维码，不推送钉钉消息')
//   //   return
//   // }

//   // if (!theConfig.dingTalkUrl) {
//   //   log('缺少 dingTalkUrl 配置，不推送钉钉消息')
//   //   return
//   // }

//   const options = {
//     alipayQRImgUrl,
//     weappQRImgUrl,
//     isExperience: theConfig.isExperience,
//     dingTalkUrl: theConfig.dingTalkUrl,
//     absConfigDir,
//     emailSetting: theConfig.emailSetting
//   }
//   await sendDing(options)
// }

function npmInstall (theConfig, absConfigDir) {
  if (!theConfig.npmInstall) {
    log('跳过 yarn install.')
    return
  }
  log('执行yarn install...')
  execSync('yarn install --silent', {
    cwd: absConfigDir,
  })
}

async function main() {
  try {
    
    const promises = []
    let alipayQRImgUrl, weappQRImgUrl

    let pullSuccess = await pullBranch()
    if (pullSuccess) {

      let absConfigPath = getAbsPath(argv.config)
      let absConfigDir = path.dirname(absConfigPath)
      
      if (!fs.existsSync(absConfigPath)) {
        log(`配置文件 ${absConfigPath} 不存在`)
        process.exit(1)
      } 
      console.log(absConfigPath)
      const defaultConfig = require('./defaultConfig')
      const theConfig = merge(defaultConfig, require(absConfigPath))
      
      if (theConfig.outDir) {
        theConfig.outDir = getAbsPath(theConfig.outDir)
        fs.ensureDirSync(theConfig.outDir)
      } 

      await npmInstall(theConfig, absConfigDir)

      if (theConfig.alipay && theConfig.alipay.enable) {
        const alipayFlow = new AlipayFlow({
          ...theConfig,
          ...theConfig.alipay,
          keyPath: getAbsPath(theConfig.alipay.keyPath),
          projectPath: getAbsPath(theConfig.alipay.projectPath),
        })
        promises.push(alipayFlow.run().then(val => alipayQRImgUrl = val))
      }
  
      if (theConfig.weapp && theConfig.weapp.enable) {
        const weappFlow = new WeappFlow({
          ...theConfig,
          ...theConfig.weapp,
          keyPath: getAbsPath(theConfig.weapp.keyPath),
          projectPath: getAbsPath(theConfig.weapp.projectPath),
        })
        promises.push(weappFlow.run().then(val => weappQRImgUrl = val))
      }
  
      await Promise.all(promises)
      // await sendDingMsg(alipayQRImgUrl, weappQRImgUrl)
      log('All done.')

    } else {
      process.exit(1)
    }

  } catch (e) {
    console.error(e)
    process.exit(1)
  }
}

main()
